unit IACubeIcon;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,extctrls,AlphaBitmapUnit,
  DirectDraw,Direct3d,D3Dx,iaregionsFC;

type
  PTriangle = ^TTriangle;
  TTriangle = array [0..3] of TD3DVERTEX;
  PCardinal = ^Cardinal;
  TIACubeIcon = class(TComponent)
  private
    { Private declarations }
    LastHeight : Integer;
    ClipRGN1 : HRGN;
    ClipRGN1FC : TIAregionFC;
    ISLoaded : Boolean;
    Ncadres : Integer;
    IconExists : Boolean;
    IconRect : TRect;
    NeedRemove : Boolean;
    ATimer : TTimer;
    FBitmap1: TBitmap;
    FIntencity: Double;
    FBitmap2: TBitmap;
    FBackColor: TColor;
    FBackBitmap: TBitmap;
    FTransparency: Double;
    LastPixelLen : Integer;
    FShowRibers: Boolean;
    FRibersColor: TColor;
    FBouncing: Boolean;
    FBaseSize: Double;
    FIconSizeCoeffitient: Double;
    FRotated: boolean;
    FYRotation: Double;
    FXrotation: double;
    FZrotation: double;
    procedure onITTimer(Sender: TObject);
    procedure DeleteX;
    procedure GetMatrixes;
    procedure InitX;
    procedure InitObjects;
    procedure InitBaseMaterials;
    function CrossProduct(V1, V2: TD3DVECTOR): TD3DVECTOR;
    function DotProduct(V1, V2: TD3DVECTOR): Double;
    function Magnitude(V: TD3DVECTOR): Double;
    procedure SetViewMatrix(var mat: TD3DMATRIX; vFrom, vAt,
      vWorldUp: TD3DVECTOR);
    procedure SetViewMatrixRelative(var mat: TD3DMATRIX; vFrom, vTO,
      vWorldUp: TD3DVECTOR);
    procedure GetCube(var ACube: array of TD3DVERTEX; x, y, z, ASideSizeX,
      ASideSizeY, ASideSizeZ: Double);
    procedure GetVolumeCube;
    function Vector(x, y, z: Double): TD3DVECTOR;
//    procedure GetTriangleFlatUV(ATriangle: PTriangle; V1, V2,
//      V3: TD3DVECTOR; dU1, dV1, dU2, dV2, dU3, dV3: Double);
    procedure CreateViewPort(ARect: TRect);
    procedure CreateClipper;
    procedure SetBitmap1(Value: TBitmap);
    procedure GetTexture1;
    procedure SetIntencity(Value: Double);
    procedure SetBitmap2(Value: TBitmap);
    procedure GetTexture2;
    procedure GetTriangleFlatUVfree(ATriangle: PTriangle; V1, V2,
      V3: TD3DVECTOR; dU1, dV1, dU2, dV2, dU3, dV3: Double);
    procedure SetBackColor(Value: TColor);
    procedure SetBackBitmap(Value: TBitmap);
    procedure SetTransparency(Value: Double);
//    procedure SetTextureAlpha(var Atex: IDirectDrawSurface7;
//      Alpha1: double);
    procedure SetTextureAlphaBitmap(var Atex: IDirectDrawSurface7;
      Alpha1: double; ABitmap: TBitmap);
    procedure SetShowRibers(Value: Boolean);
    procedure SetRibersColor(Value: TColor);
    procedure SetBouncing(Value: Boolean);
    procedure SetBaseSize(Value: Double);
    procedure SetIconSizeCoeffitient(Value: Double);
    function AntiColor(AColor : TColor): TColor;
    procedure SetNewClipper;
    function GetVisiblePartofRect(Const ARect : TRect): HRGN;
    function UNVisiblePartofRect(const ARect : TRect): HRGN;
    procedure SetRotated(Value: boolean);
    procedure SetXrotation(Value: double);
    procedure SetYRotation(Value: Double);
    procedure SetZrotation(Value: double);

  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Loaded; override;
  published
    { Published declarations }
    property Bitmap1 : TBitmap read FBitmap1 write SetBitmap1;
    property Bitmap2 : TBitmap read FBitmap2 write SetBitmap2;
    property Intencity : Double read FIntencity write SetIntencity;
    property Transparency : Double read FTransparency write SetTransparency;
    property BackColor : TColor read FBackColor write SetBackColor default clBlack;
    property BackBitmap : TBitmap read FBackBitmap write SetBackBitmap;
    property ShowRibers : Boolean read FShowRibers write SetShowRibers default TRUE;
    property RibersColor : TColor read FRibersColor write SetRibersColor default clBlack;
    property Bouncing : Boolean read FBouncing write SetBouncing default false;
    property BaseSize : Double read FBaseSize write SetBaseSize;
    property IconSizeCoeffitient : Double read FIconSizeCoeffitient write SetIconSizeCoeffitient;
    property Rotated : boolean read FRotated write SetRotated default true;
    property Xrotation : double read FXrotation write SetXrotation;
    property YRotation : Double read FYRotation write SetYRotation;
    property Zrotation : double read FZrotation write SetZrotation;
  end;

procedure Register;
function EnumZBufferCallback(var lpDDPixFmt: TDDPixelFormat;
      lpContext: Pointer): HResult; stdcall;

var
  CurrDD : IDirectDraw7;
  Curr3D : IDirect3D7;
  Curr3DDevice : IDirect3DDevice7;
  PrimaryBuff,BackBuff,ZBuff : IDirectDrawSurface7;
  CurrWorldMatrix : TD3DMATRIX;
  CurrViewMatrix : TD3DMATRIX;
  CurrProjectionMatrix : TD3DMATRIX;
  V1View,V2View,V3View : TD3DVECTOR;
  BaseMaterial1,RibersMaterial : TD3DMATERIAL7;
  AMainVolume : array [0..35] of TD3DVERTEX;
  ARibers : array [0..23]  of TD3DVERTEX;
  AIconSize : Integer;
  XTexture1,XTexture2 : IDirectDrawSurface7;
  MainDC : HDC;

implementation

procedure Register;
begin
  RegisterComponents('TIA', [TIACubeIcon]);
end;

{ TIACubeIcon }

constructor TIACubeIcon.Create(AOwner: TComponent);
var  i,N,dW,dH : integer;
begin
  inherited Create(AOwner);
  FXRotation:=0.05;
  FYRotation:=0.1;
  FZRotation:=0.0;
  FRotated:=true;
  ClipRGN1:=CreateRectRGN(0,0,0,0);
  ClipRGN1FC := TIAregionFC.Create(self);
  ClipRGN1FC.Region:=ClipRGN1;
  ISLoaded:=false;
  FIconSizeCoeffitient:=1;
  FBaseSize:=1;
  FBouncing:=false;   Ncadres:=0;
  FRibersColor:=clBlack;
  RibersMaterial.emissive.r :=0;
  RibersMaterial.emissive.g :=  0;
  RibersMaterial.emissive.b :=  0;
  RibersMaterial.diffuse.a:=0.0;
  FShowRibers:=TRUE;
  LastPixelLen:=0;
  FBackColor:=clBlack;
  FIntencity:=1;
  FTransparency:=0;
  FBitmap1:=TBitmap.Create;
  FBitmap1.Width:=0;
  FBitmap1.Height:=0;
  FBitmap1.PixelFormat:=pf24Bit;
  FBitmap2:=TBitmap.Create;
  FBitmap2.Width:=0;
  FBitmap2.Height:=0;
  FBitmap2.PixelFormat:=pf24Bit;
  FBackBitmap:=TBitmap.Create;
  FBackBitmap.Width:=0;
  FBackBitmap.Height:=0;
  FBackBitmap.PixelFormat:=pf24Bit;
  IconExists:=false;
  NeedRemove:=false;
  AIconSize:=0;
  if (AOwner<>nil) and (AOwner is TForm) then
    begin
      N:=0;
      for i:=0 to (AOwner as TForm).ComponentCount-1 do
        begin
          if (AOwner as TForm).Components[i] is TIACubeIcon then inc(N);
        end;
      if N>1 then
        begin
          ShowMessage('No more then one TIACubeIcon allowed');
          NeedRemove:=True;
        end else IconExists:=TRUE;


        dw:=((AOwner as TForm).width-(AOwner as TForm).clientwidth) div 2;
        dh:=(AOwner as TForm).height-(AOwner as TForm).clientheight-dw;
        AIconSize:=round((dh-dw div 2)*FIconSizeCoeffitient);
        if AIconSize<0 then AIconSize:=1;
        IConRect:=rect(dw div 2,dw div 2,dw div 2+AIconSize,dw div 2+AIconSize);
        LastHeight:=AIconSize;

    end;
  D3DXInitialize;

  ATimer:=TTimer.Create(Self);
  ATimer.Interval:=100;
  ATimer.OnTimer:=onITTimer;
  if not NeedRemove then
    begin
      onITTimer(Self);
    end;

end;

destructor TIACubeIcon.Destroy;
begin

  FBitmap1.free;
  FBitmap2.free;
  FBackBitmap.free;
  ATimer.Enabled:=false;
  ATimer.free;
  deleteobject(ClipRGN1);
  ClipRGN1FC.Free;
  DeleteX;
  D3DXUninitialize;
  inherited Destroy;
end;

procedure TIACubeIcon.onITTimer(Sender: TObject);
var
  ToRect,FromRect : TD3DRect;
  ARotmat : TD3DMATRIX;
  dw,dh : Integer;
  Torect1,FromRect1,Torect2 : TRect;
  DC,DC1 :HDC;
  dx,dy : Integer;

begin
  if NeedRemove then
    begin
      Destroy;
    end else
    begin
      ATimer.Enabled:=false;
      if CurrDD=nil then
        begin
          InitX;
          if CurrDD=nil then exit;
        end else
        begin
          if (Curr3DDevice<>nil) then
            begin
                  dw:=((Owner as TForm).width-(Owner as TForm).clientwidth) div 2;
                  dh:=(Owner as TForm).height-(Owner as TForm).clientheight-dw;
                  AIconSize:=round((dh-dw div 2)*FIconSizeCoeffitient);
                  if AIconSize<0 then AIconSize:=1;
                  IConRect:=rect(dw div 2,dw div 2,dw div 2+AIconSize,dw div 2+AIconSize);
                  ToRect.x1:=IconRect.left+(Owner as TForm).left;
                  ToRect.y1:=IconRect.top+(Owner as TForm).top;
                  ToRect.x2:=IconRect.right+(Owner as TForm).left;
                  ToRect.y2:=IconRect.bottom+(Owner as TForm).top;
                  Torect1:=IconRect;
                  dx:=(Owner as TForm).left;
                  dy:=(Owner as TForm).top;
                  if LastHeight<>AIconSize then
                    begin
                      DeleteX;
                      InitX;
                      GetTexture1;
                      GetTexture2;
                    end;
                  LastHeight:=AIconSize;

              FromRect.x1:=0;
              FromRect.y1:=0;
              FromRect.x2:=AIconSize;
              FromRect.y2:=AIconSize;
              FromRect1:=rect(0,0,BackBitmap.width,BackBitmap.height);
              Torect2:=rect(FromRect.x1,FromRect.y1,FromRect.x2,FromRect.y2);

              DC:=getwindowdc((Owner as TForm).handle);

              Curr3DDevice.Clear( 0, nil, D3DCLEAR_TARGET or D3DCLEAR_ZBUFFER ,AntiColor(Colortorgb(FBackColor)),0, 0);
              if  FBackBitmap.width<>0 then
                begin
                  BackBuff.GetDC(DC1);
                  stretchblt(DC1,0,0,AIconSize,AIconSize,FBackBitmap.canvas.handle,0,0,FBackBitmap.width,FBackBitmap.height,SRCCOPY);
                  BackBuff.ReleaseDC(DC1);
                end;

              if RectVisible(DC,Torect1) then
                begin
                  deleteobject(ClipRGN1);
                  ClipRgn1:=GetVisiblePartofRect(IConRect);
                  offsetrgn(ClipRgn1,dx,dy);
                  ClipRGN1FC.Region:=ClipRGN1;
                  SetNewClipper;
                  if FRotated then
                    begin
                      D3DXMatrixRotationX(ARotmat,FXRotation);
                      D3DXMatrixMultiply(CurrWorldMatrix,CurrWorldMatrix,ARotmat);
                      D3DXMatrixRotationY(ARotmat,FYRotation);
                      D3DXMatrixMultiply(CurrWorldMatrix,CurrWorldMatrix,ARotmat);
                      D3DXMatrixRotationZ(ARotmat,FZRotation);
                      D3DXMatrixMultiply(CurrWorldMatrix,CurrWorldMatrix,ARotmat);
                      Curr3DDevice.SetTransform( D3DTRANSFORMSTATE_WORLD, CurrWorldMatrix);
                    end;
                  if FBouncing then
                    begin
                      V1View.x:=0;V1View.y:=0;V1View.z:=-15+5*cos(Ncadres/15);
                      SetViewMatrixRelative(CurrViewMatrix,V1View,V2View,V3View);
                      Curr3DDevice.SetTransform( D3DTRANSFORMSTATE_VIEW, CurrViewMatrix);
                      if Ncadres>100000 then Ncadres:=0 else inc(Ncadres);
                    end;

                  Curr3DDevice.BeginScene;

                  Curr3DDevice.SetMaterial(BaseMaterial1);
                  if FBitmap1.width<>0 then
                    begin
                      Curr3DDevice.SetTexture(0,XTexture1);
                      Curr3DDevice.SetTextureStageState( 0, D3DTSS_COLOROP, Cardinal(D3DTOP_MODULATE) );
                      Curr3DDevice.SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_TEXTURE );
                      Curr3DDevice.SetTextureStageState( 0, D3DTSS_COLORARG2, D3DTA_CURRENT );
                      Curr3DDevice.SetTextureStageState( 0, D3DTSS_ALPHAOP, Cardinal(D3DTOP_SELECTARG1) );
                      Curr3DDevice.SetTextureStageState( 0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE );
                      Curr3DDevice.DrawPrimitive(D3DPT_TRIANGLELIST, D3DFVF_VERTEX,
                                              AMainVolume[0], 18, 0 );
                      Curr3DDevice.SetTexture(0,nil);
                    end else
                    begin
                      Curr3DDevice.DrawPrimitive(D3DPT_TRIANGLELIST, D3DFVF_VERTEX,
                                              AMainVolume[0], 36, 0 );
                    end;
                  if FBitmap2.width<>0 then
                    begin
                      Curr3DDevice.SetTexture(0,XTexture2);
                      Curr3DDevice.SetTextureStageState( 0, D3DTSS_COLOROP, Cardinal(D3DTOP_MODULATE) );
                      Curr3DDevice.SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_TEXTURE );
                      Curr3DDevice.SetTextureStageState( 0, D3DTSS_COLORARG2, D3DTA_CURRENT );
                      Curr3DDevice.SetTextureStageState( 0, D3DTSS_ALPHAOP, Cardinal(D3DTOP_SELECTARG1) );
                      Curr3DDevice.SetTextureStageState( 0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE );
                      Curr3DDevice.DrawPrimitive(D3DPT_TRIANGLELIST, D3DFVF_VERTEX,
                                              AMainVolume[18], 18, 0 );
                      Curr3DDevice.SetTexture(0,nil);
                    end else
                    begin
                      if FBitmap1.width<>0 then
                        begin
                          Curr3DDevice.SetTexture(0,XTexture1);
                          Curr3DDevice.SetTextureStageState( 0, D3DTSS_COLOROP, Cardinal(D3DTOP_MODULATE) );
                          Curr3DDevice.SetTextureStageState( 0, D3DTSS_COLORARG1, D3DTA_TEXTURE );
                          Curr3DDevice.SetTextureStageState( 0, D3DTSS_COLORARG2, D3DTA_CURRENT );
                          Curr3DDevice.SetTextureStageState( 0, D3DTSS_ALPHAOP, Cardinal(D3DTOP_SELECTARG1) );
                          Curr3DDevice.SetTextureStageState( 0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE );
                          Curr3DDevice.DrawPrimitive(D3DPT_TRIANGLELIST, D3DFVF_VERTEX,
                                                  AMainVolume[18], 18, 0 );
                          Curr3DDevice.SetTexture(0,nil);
                        end;  
                    end;  

                  if  FShowRibers then
                    begin
                      Curr3DDevice.SetMaterial(RibersMaterial);
                      Curr3DDevice.DrawPrimitive(D3DPT_LINELIST, D3DFVF_VERTEX,
                                              ARibers[0], 24, 0 );

                    end;
                  Curr3DDevice.EndScene;

                 if  failed(PrimaryBuff.Blt(@ToRect, BackBuff,@FromRect, DDBLT_WAIT, nil )) then
                   begin
//                     PrimaryBuff._Restore;
  //                   BackBuff._Restore;
    //                 ZBuff._Restore;
                     DeleteX;
                     InitX;
                     GetTexture1;
                     GetTexture2;
                   end;
                 end;
               releasedc((Owner as TForm).handle,DC);
            end;
        end;
      ATimer.Enabled:=TRUE;
    end;
end;
procedure TIACubeIcon.DeleteX;
begin
  if XTexture2<>nil then XTexture2:=nil;
  if XTexture1<>nil then XTexture1:=nil;
  if ZBuff<>nil then  ZBuff:=nil;
  if BackBuff<>nil then  BackBuff:=nil;
  if PrimaryBuff<>nil then  PrimaryBuff:=nil;
  if Curr3DDevice<>nil then  Curr3DDevice:=nil;
  if Curr3D<>nil then Curr3D:=nil;
  if CurrDD<>nil then  CurrDD:=nil;
end;
procedure TIACubeIcon.InitX;
var 
    ddsd : TDDSURFACEDESC2;
    ddpfZBuffer : TDDPIXELFORMAT;
    HardResolve : Boolean;
begin
  //3D
  if (Owner<>nil) and (Owner is TForm) then
    begin
      //recreation of surfaces...
      if failed(DirectDrawCreateEx( nil, CurrDD, IID_IDirectDraw7, nil )) then
        begin
          showmessage('Can''t create DirectDraw object, failed');
          exit;
        end;
      if CurrDD=nil then exit;
      CurrDD.SetCooperativeLevel( (Owner as TForm).handle, DDSCL_NORMAL );

      ZeroMemory( @ddsd, sizeof(TDDSURFACEDESC2) );
      ddsd.dwSize         := sizeof(TDDSURFACEDESC2);
      ddsd.dwFlags        := DDSD_CAPS;
      ddsd.ddsCaps.dwCaps := DDSCAPS_PRIMARYSURFACE;

      // Create the primary surface.
      CurrDD.CreateSurface( ddsd, PrimaryBuff, nil );
      ddsd.dwFlags        := DDSD_WIDTH or DDSD_HEIGHT or DDSD_CAPS;
      ddsd.ddsCaps.dwCaps := DDSCAPS_OFFSCREENPLAIN or DDSCAPS_3DDEVICE;
      ddsd.dwWidth  := AIconSize;
      ddsd.dwHeight := AIconSize;

    // Create the backbuffer. The most likely reason for failure is running
    // out of video memory. (A more sophisticated app should handle this.)
      CurrDD.CreateSurface( ddsd, BackBuff, nil );
      CurrDD.QueryInterface( IID_IDirect3D7, Curr3D );
      if Curr3D=nil then exit;
      HardResolve:=true;
      if failed(Curr3D.EnumZBufferFormats( IID_IDirect3DHALDevice,
                                 EnumZBufferCallback, @ddpfZBuffer )) then
        begin
          HardResolve:=false;
          if failed(Curr3D.EnumZBufferFormats( IID_IDirect3DRGBDevice,
                                 EnumZBufferCallback, @ddpfZBuffer )) then
            begin
              showmessage('Can''t to create any 3D Device, halted');
              halt;
            end;
        end;


      ddsd.dwFlags        := DDSD_CAPS or DDSD_WIDTH or DDSD_HEIGHT or DDSD_PIXELFORMAT;
      if HardResolve then
      ddsd.ddsCaps.dwCaps := DDSCAPS_ZBUFFER or DDSCAPS_VIDEOMEMORY
      else
      ddsd.ddsCaps.dwCaps := DDSCAPS_ZBUFFER or DDSCAPS_SYSTEMMEMORY;
      ddsd.dwWidth        := AIconSize;
      ddsd.dwHeight       := AIconSize;
      copymemory( @(ddsd.ddpfPixelFormat), @ddpfZBuffer, sizeof(TDDPIXELFORMAT) );

    // Create and attach a z-buffer. Real apps should be able to handle an
    // error here (DDERR_OUTOFVIDEOMEMORY may be encountered). For this
    // tutorial, though, we are simply going to exit ungracefully.
      if failed(CurrDD.CreateSurface( ddsd, ZBuff, nil )) and  HardResolve then
        begin
          //Try to move to system memory
          ddsd.dwFlags        := DDSD_CAPS or DDSD_WIDTH or DDSD_HEIGHT or DDSD_PIXELFORMAT;
          ddsd.ddsCaps.dwCaps := DDSCAPS_ZBUFFER or DDSCAPS_SYSTEMMEMORY;
          ddsd.dwWidth        := AIconSize;
          ddsd.dwHeight       := AIconSize;
          copymemory( @(ddsd.ddpfPixelFormat), @ddpfZBuffer, sizeof(TDDPIXELFORMAT) );
          if failed(CurrDD.CreateSurface( ddsd, ZBuff, nil )) then
            begin
              showmessage('No Z buffer so no fine draw');
            end;
        end;
      BackBuff.AddAttachedSurface( ZBuff );


      ddsd.dwSize := sizeof(TDDSURFACEDESC2);
    // The GUID here is hard coded. In a real-world application
    // this should be retrieved by enumerating devices.
      Curr3D.CreateDevice( IID_IDirect3DHALDevice, BackBuff,Curr3DDevice );
      if Curr3DDevice=nil then
        begin
          showmessage('Can''t to create D3DDevice object, failed');
          exit;
        end;
        //Toenable Z Buffering...
      Curr3DDevice.SetRenderState(D3DRENDERSTATE_ZENABLE,cardinal(D3DZB_USEW));
      Curr3DDevice.SetRenderState(D3DRENDERSTATE_ZVISIBLE,cardinal(D3DZB_TRUE));
      Curr3DDevice.SetRenderState(D3DRENDERSTATE_ZWRITEENABLE,cardinal(D3DZB_TRUE));
      Curr3DDevice.SetRenderState(D3DRENDERSTATE_DITHERENABLE,cardinal(D3DZB_TRUE));
      Curr3DDevice.SetRenderState( D3DRENDERSTATE_AMBIENT, $00ffffff );
       Curr3DDevice.SetRenderState(D3DRENDERSTATE_TEXTUREPERSPECTIVE,1);
      Curr3DDevice.SetRenderState(D3DRENDERSTATE_SPECULARMATERIALSOURCE,cardinal(D3DMCS_MATERIAL ));
      Curr3DDevice.SetRenderState(D3DRENDERSTATE_DIFFUSEMATERIALSOURCE ,cardinal(D3DMCS_MATERIAL ));
      Curr3DDevice.SetRenderState(D3DRENDERSTATE_AMBIENTMATERIALSOURCE ,cardinal(D3DMCS_MATERIAL ));
      Curr3DDevice.SetRenderState(D3DRENDERSTATE_EMISSIVEMATERIALSOURCE ,cardinal(D3DMCS_MATERIAL ));
      Curr3DDevice.SetRenderState( D3DRENDERSTATE_SHADEMODE, cardinal(D3DSHADE_PHONG     ));
      Curr3DDevice.SetRenderState( D3DRENDERSTATE_ALPHABLENDENABLE, 1 );
      Curr3DDevice.SetRenderState( D3DRENDERSTATE_COLORKEYENABLE, 1 );
      Curr3DDevice.SetRenderState( D3DRENDERSTATE_SRCBLEND, cardinal(D3DBLEND_INVSRCALPHA         ));
      Curr3DDevice.SetRenderState( D3DRENDERSTATE_DESTBLEND, cardinal(D3DBLEND_SRCALPHA          ));
      //Resize
      CreateClipper;
      CreateViewPort(IconRect);
      GetMatrixes;
      InitObjects;
      InitBaseMaterials;
    end;
end;
procedure TIACubeIcon.GetMatrixes;

begin
//FillWorldMatrix
  D3DXMatrixIdentity(CurrWorldMatrix);
//FillViewMatrix;
  V1View.x:=0;V1View.y:=0;V1View.z:=-10;
  V2View.x:=0.0;V2View.y:=0.0;V2View.z:=1.0;
  V3View.x:=0;V3View.y:=1;V3View.z:=0;
  SetViewMatrixRelative(CurrViewMatrix,V1View,V2View,V3View);
//FillProjectionMetrix;
  D3DXMatrixPerspectiveLH(CurrProjectionMatrix,1*FBaseSize,1*FBaseSize,6,24);
//setting
  Curr3DDevice.SetTransform( D3DTRANSFORMSTATE_WORLD, CurrWorldMatrix);
  Curr3DDevice.SetTransform( D3DTRANSFORMSTATE_VIEW, CurrViewMatrix);
  Curr3DDevice.SetTransform( D3DTRANSFORMSTATE_PROJECTION, CurrProjectionMatrix);
end;
procedure TIACubeIcon.InitObjects;
begin
  GetVolumeCube;
end;
procedure TIACubeIcon.InitBaseMaterials;
begin
  BaseMaterial1.emissive.r := 1.0;
  BaseMaterial1.emissive.g := 1.0;
  BaseMaterial1.emissive.b := 1.0;
  BaseMaterial1.diffuse.a:=0.0;
end;
//------
procedure TIACubeIcon.SetViewMatrixRelative(var mat : TD3DMATRIX;vFrom : TD3DVECTOR;vTO : TD3DVECTOR;vWorldUp : TD3DVECTOR);
begin
  SetViewMatrix(mat,vFrom,D3DXVec3Add(vFrom,vTO),vWorldUp);
end;
procedure TIACubeIcon.SetViewMatrix(var mat : TD3DMATRIX;vFrom : TD3DVECTOR;vAt : TD3DVECTOR;vWorldUp : TD3DVECTOR);
var
  vView,vUP,vRight : TD3DVECTOR;
  fLength,fDotProduct : Double;
begin
  // Get the z basis vector, which points straight ahead. This is the
  // difference from the eyepoint to the lookat point.
  vView.x := vAt.x - vFrom.x;
  vView.y := vAt.y - vFrom.y;
  vView.z := vAt.z - vFrom.z;

  fLength := Magnitude(vView);
  if fLength < (1.0e-6) then exit;
  // Normalize the z basis vector
  vView.x:=vView.x/fLength;
  vView.y:=vView.y/fLength;
  vView.z:=vView.z/fLength;
  // Get the dot product, and calculate the projection of the z basis
  // vector onto the up vector. The projection is the y basis vector.
  fDotProduct := DotProduct( vWorldUp, vView );
  vUp.x:=vWorldUp.x-fDotProduct*vView.x;
  vUp.y:=vWorldUp.y-fDotProduct*vView.y;
  vUp.z:=vWorldUp.z-fDotProduct*vView.z;

  // Normalize the y basis vector
  vUp.x:=vUp.x/fLength;
  vUp.y:=vUp.y/fLength;
  vUp.z:=vUp.z/fLength;
  // The x basis vector is found simply with the cross product of the y
  // and z basis vectors
  vRight := CrossProduct( vUp, vView );
  // Start building the matrix. The first three rows contains the basis
  // vectors used to rotate the view to point at the lookat point
  D3DXMatrixIdentity( mat );
  mat._11 := vRight.x;    mat._12 := vUp.x;    mat._13 := vView.x;
  mat._21 := vRight.y;    mat._22 := vUp.y;    mat._23 := vView.y;
  mat._31 := vRight.z;    mat._32 := vUp.z;    mat._33 := vView.z;
 
  // Do the translation values (rotations are still about the eyepoint)
  mat._41 := - DotProduct( vFrom, vRight );
  mat._42 := - DotProduct( vFrom, vUp );
  mat._43 := - DotProduct( vFrom, vView );
end;
function  TIACubeIcon.Magnitude(V : TD3DVECTOR): Double;
begin
  result:=sqrt(sqr(V.x)+sqr(V.y)+sqr(V.z));
end;
function  TIACubeIcon.DotProduct(V1,V2 : TD3DVECTOR): Double;
begin
  result:=V1.x*V2.x+V1.y*V2.y+V1.z*V2.z;
end;
function  TIACubeIcon.CrossProduct(V1,V2 : TD3DVECTOR): TD3DVECTOR;
begin
  result.x:=V1.y*V2.z-V1.z*V2.y;
  result.y:=V1.z*V2.x-V1.x*V2.z;
  result.z:=V1.x*V2.y-V1.y*V2.x;
end;
procedure TIACubeIcon.GetVolumeCube;
var i : integer;
begin
  GetCube(AMainVolume,0,0,0,1,1,1);
  for i:=0 to 23 do
    begin
      ARibers[i].nx:=0;ARibers[i].ny:=0;ARibers[i].nz:=0;
      ARibers[i].tu:=0;  ARibers[i].tv:=0;
    end;
  ARibers[0].x:=-0.5; ARibers[0].y:=-0.5; ARibers[0].z:=-0.5;
  ARibers[1].x:= 0.5; ARibers[1].y:=-0.5; ARibers[1].z:=-0.5;
  ARibers[2].x:=-0.5; ARibers[2].y:=-0.5; ARibers[2].z:=-0.5;
  ARibers[3].x:=-0.5; ARibers[3].y:= 0.5; ARibers[3].z:=-0.5;
  ARibers[4].x:= 0.5; ARibers[4].y:= 0.5; ARibers[4].z:=-0.5;
  ARibers[5].x:=-0.5; ARibers[5].y:= 0.5; ARibers[5].z:=-0.5;
  ARibers[6].x:= 0.5; ARibers[6].y:= 0.5; ARibers[6].z:=-0.5;
  ARibers[7].x:= 0.5; ARibers[7].y:=-0.5; ARibers[7].z:=-0.5;

  ARibers[8].x:=-0.5; ARibers[8].y:=-0.5; ARibers[8].z:= 0.5;
  ARibers[9].x:= 0.5; ARibers[9].y:=-0.5; ARibers[9].z:= 0.5;
  ARibers[10].x:=-0.5;ARibers[10].y:=-0.5;ARibers[10].z:= 0.5;
  ARibers[11].x:=-0.5;ARibers[11].y:= 0.5;ARibers[11].z:=0.5;
  ARibers[12].x:= 0.5;ARibers[12].y:= 0.5;ARibers[12].z:= 0.5;
  ARibers[13].x:= 0.5;ARibers[13].y:=-0.5;ARibers[13].z:= 0.5;
  ARibers[14].x:= 0.5;ARibers[14].y:= 0.5;ARibers[14].z:= 0.5;
  ARibers[15].x:=-0.5;ARibers[15].y:= 0.5;ARibers[15].z:= 0.5;

  ARibers[16].x:=-0.5;ARibers[16].y:=-0.5;ARibers[16].z:=-0.5;
  ARibers[17].x:=-0.5;ARibers[17].y:=-0.5;ARibers[17].z:= 0.5;
  ARibers[18].x:=-0.5;ARibers[18].y:= 0.5;ARibers[18].z:=-0.5;
  ARibers[19].x:=-0.5;ARibers[19].y:= 0.5;ARibers[19].z:= 0.5;
  ARibers[20].x:= 0.5;ARibers[20].y:=-0.5;ARibers[20].z:=-0.5;
  ARibers[21].x:= 0.5;ARibers[21].y:=-0.5;ARibers[21].z:= 0.5;
  ARibers[22].x:= 0.5;ARibers[22].y:= 0.5;ARibers[22].z:=-0.5;
  ARibers[23].x:= 0.5;ARibers[23].y:= 0.5;ARibers[23].z:= 0.5;
end;

procedure TIACubeIcon.GetCube(var ACube : array of TD3DVERTEX;x,y,z,ASideSizeX,ASideSizeY,ASideSizeZ : Double);
var
  p1,p2,p3,p4,p5,p6,p7,p8 : TD3DVECTOR;
  Amin,AMax: integer;
begin
  AMin:=low(ACube);
  AMax:=high(ACube);
  if (AMax-AMin)<35 then exit;
  p1:=Vector(x+ASideSizeX/2,y-ASideSizeY/2,z-ASideSizeZ/2);
  p2:=Vector(x-ASideSizeX/2,y-ASideSizeY/2,z-ASideSizeZ/2);
  p3:=Vector(x+ASideSizeX/2,y+ASideSizeY/2,z-ASideSizeZ/2);
  p4:=Vector(x-ASideSizeX/2,y+ASideSizeY/2,z-ASideSizeZ/2);
  p5:=Vector(x+ASideSizeX/2,y-ASideSizeY/2,z+ASideSizeZ/2);
  p6:=Vector(x-ASideSizeX/2,y-ASideSizeY/2,z+ASideSizeZ/2);
  p7:=Vector(x+ASideSizeX/2,y+ASideSizeY/2,z+ASideSizeZ/2);
  p8:=Vector(x-ASideSizeX/2,y+ASideSizeY/2,z+ASideSizeZ/2);
    //ACubeInfo
 { GetTriangleFlatUV(PTriangle(@(ACube[0+AMin])),p1,p2,p3,0,0,1,0,0,1);
  GetTriangleFlatUV(PTriangle(@(ACube[3+AMin])),p2,p4,p3,1,0,1,1,0,1);
  GetTriangleFlatUV(PTriangle(@(ACube[6+AMin])),p5,p7,p6,1,0,1,1,0,0);
  GetTriangleFlatUV(PTriangle(@(ACube[9+AMin])),p6,p7,p8,0,0,1,1,0,1);
  GetTriangleFlatUV(PTriangle(@(ACube[12+AMin])),p1,p5,p2,0,0,1,0,0,1);
  GetTriangleFlatUV(PTriangle(@(ACube[15+AMin])),p5,p6,p2,1,0,1,1,0,1);
  GetTriangleFlatUV(PTriangle(@(ACube[18+AMin])),p2,p6,p4,0,0,1,0,0,1);
  GetTriangleFlatUV(PTriangle(@(ACube[21+AMin])),p6,p8,p4,1,0,1,1,0,1);
  GetTriangleFlatUV(PTriangle(@(ACube[24+AMin])),p4,p8,p3,0,0,1,0,0,1);
  GetTriangleFlatUV(PTriangle(@(ACube[27+AMin])),p8,p7,p3,1,0,1,1,0,1);
  GetTriangleFlatUV(PTriangle(@(ACube[30+AMin])),p3,p7,p1,0,0,1,0,0,1);
  GetTriangleFlatUV(PTriangle(@(ACube[33+AMin])),p7,p5,p1,1,0,1,1,0,1);  }

  //GetTriangleFlatUVfree
  GetTriangleFlatUVfree(PTriangle(@(ACube[0+AMin])),p1,p2,p3,0,0,1,0,0,1);
  GetTriangleFlatUVfree(PTriangle(@(ACube[3+AMin])),p2,p4,p3,1,0,1,1,0,1);
  GetTriangleFlatUVfree(PTriangle(@(ACube[6+AMin])),p5,p7,p6,1,0,1,1,0,0);
  GetTriangleFlatUVfree(PTriangle(@(ACube[9+AMin])),p6,p7,p8,0,0,1,1,0,1);
  GetTriangleFlatUVfree(PTriangle(@(ACube[12+AMin])),p1,p5,p2,0,0,1,0,0,1);
  GetTriangleFlatUVfree(PTriangle(@(ACube[15+AMin])),p5,p6,p2,1,0,1,1,0,1);
  GetTriangleFlatUVfree(PTriangle(@(ACube[18+AMin])),p2,p6,p4,0,0,1,0,0,1);
  GetTriangleFlatUVfree(PTriangle(@(ACube[21+AMin])),p6,p8,p4,1,0,1,1,0,1);
  GetTriangleFlatUVfree(PTriangle(@(ACube[24+AMin])),p4,p8,p3,0,0,1,0,0,1);
  GetTriangleFlatUVfree(PTriangle(@(ACube[27+AMin])),p8,p7,p3,1,0,1,1,0,1);
  GetTriangleFlatUVfree(PTriangle(@(ACube[30+AMin])),p3,p7,p1,0,0,1,0,0,1);
  GetTriangleFlatUVfree(PTriangle(@(ACube[33+AMin])),p7,p5,p1,1,0,1,1,0,1);
end;
function TIACubeIcon.Vector(x, y, z: Double): TD3DVECTOR;
begin
  Result.x:=x;
  Result.y:=y;
  Result.z:=z;
end;
{procedure TIACubeIcon.GetTriangleFlatUV(ATriangle: PTriangle; V1, V2,
  V3: TD3DVECTOR; dU1, dV1,dU2,dV2,dU3,dV3: Double);
  var  Va,Vb,n : TD3DVECTOR;
begin
  Va:=D3DXVec3Subtract(V2,V1);
  Vb:=D3DXVec3Subtract(V3,V1);
  D3DXVec3Cross(n,Va,Vb);
  D3DXVec3Normalize(n,n);

  ATriangle[0].x:=V1.x;ATriangle[0].y:=V1.y;ATriangle[0].z:=V1.z;
  ATriangle[ 0].nx:=n.x;ATriangle[ 0].ny:=n.y;ATriangle[ 0].nz:=n.z;
  ATriangle[ 0].tu:=dU1;  ATriangle[ 0].tv:=dV1;
  ATriangle[ 1].x:=V2.x;ATriangle[ 1].y:=V2.y;ATriangle[ 1].z:=V2.z;
  ATriangle[ 1].nx:=n.x;ATriangle[ 1].ny:=n.y;ATriangle[ 1].nz:=n.z;
  ATriangle[ 1].tu:=dU2;  ATriangle[ 1].tv:=dV2;
  ATriangle[ 2].x:=V3.x;ATriangle[ 2].y:=V3.y;ATriangle[ 2].z:=V3.z;
  ATriangle[ 2].nx:=n.x;ATriangle[ 2].ny:=n.y;ATriangle[ 2].nz:=n.z;
  ATriangle[ 2].tu:=dU3;  ATriangle[ 2].tv:=dV3;
end;}
procedure TIACubeIcon.GetTriangleFlatUVfree(ATriangle: PTriangle; V1, V2,
  V3: TD3DVECTOR; dU1, dV1,dU2,dV2,dU3,dV3: Double);

begin


  ATriangle[0].x:=V1.x;ATriangle[0].y:=V1.y;ATriangle[0].z:=V1.z;
  ATriangle[ 0].nx:=0;ATriangle[ 0].ny:=0;ATriangle[ 0].nz:=0;
  ATriangle[ 0].tu:=dU1;  ATriangle[ 0].tv:=dV1;
  ATriangle[ 1].x:=V2.x;ATriangle[ 1].y:=V2.y;ATriangle[ 1].z:=V2.z;
  ATriangle[ 1].nx:=0;ATriangle[ 1].ny:=0;ATriangle[ 1].nz:=0;
  ATriangle[ 1].tu:=dU2;  ATriangle[ 1].tv:=dV2;
  ATriangle[ 2].x:=V3.x;ATriangle[ 2].y:=V3.y;ATriangle[ 2].z:=V3.z;
  ATriangle[ 2].nx:=0;ATriangle[ 2].ny:=0;ATriangle[ 2].nz:=0;
  ATriangle[ 2].tu:=dU3;  ATriangle[ 2].tv:=dV3;
end;
procedure TIACubeIcon.CreateViewPort(ARect : TRect);
var
  ViewportData : TD3DVIEWPORT7;
begin
  //Define the Viewport data
  ViewportData.dwX:=-AIconSize div 2;
  ViewportData.dwY:=-AIconSize div 2;
  ViewportData.dwWidth:=AIconSize;
  ViewportData.dwHeight:=AIconSize;
  ViewportData.dvMinZ:=0.0;
  ViewportData.dvMaxZ:=1.0;
  Curr3DDevice.SetViewport(ViewPortData);
end;
procedure TIACubeIcon.CreateClipper;
//var
//  TempClipper : IDIRECTDRAWCLIPPER;
begin
{  //we create Clipper for surfaces...
  CurrDD.CreateClipper(0,TempClipper,nil);
  TempClipper.SetHWnd(0,(owner as TForm).handle);
  //setting clipper to primary surface...
  PrimaryBuff.SetClipper(TempClipper);
  TempClipper:=nil;//no need more, after assigning...  }
end;
procedure TIACubeIcon.SetNewClipper;
  var
    TempClipper : IDIRECTDRAWCLIPPER;

begin
  //we create Clipper for surfaces...
  CurrDD.CreateClipper(0,TempClipper,nil);
  if failed(TempClipper.SetClipList(ClipRGN1FC.RegionData,0)) then
    showmessage('Clipping failed');
  //setting clipper to primary surface...
  PrimaryBuff.SetClipper(TempClipper);
  TempClipper:=nil;//no need more, after assigning...  
end;
function EnumZBufferCallback(var lpDDPixFmt: TDDPixelFormat;
      lpContext: Pointer): HResult; stdcall;
begin

    if( lpDDPixFmt.dwFlags = DDPF_ZBUFFER ) then
      begin

        copymemory( lpContext, @lpDDPixFmt, sizeof(TDDPIXELFORMAT) );

        result:=D3DENUMRET_CANCEL;
     end else result:=D3DENUMRET_OK;

end;
procedure TIACubeIcon.SetBitmap1(Value: TBitmap);
begin
  if (Value=nil) or (Value.width=0) or (Value.height=0) then
    begin
      FBitmap1.width:=0;
      FBitmap1.height:=0;
      if Curr3DDevice<>nil then
      GetTexture1;
    end else
    begin
      FBitmap1.width:=Value.width;
      FBitmap1.height:=Value.height;
      FBitmap1.Canvas.Draw(0,0,Value);
      if Curr3DDevice<>nil then
      GetTexture1;
    end;
end;
procedure TIACubeIcon.GetTexture1;
  var
    flags1,width1,height1,num1 :DWord;
    flags2 : TD3DX_SurfaceFormat;
    mypallete : IDirectDrawPalette;
    Wnd : HWND;
    DC : HDC;
    BPP : Integer;
begin
  if Curr3DDevice=nil then exit;
  Wnd:=GetDesktopWindow;
  DC:=getwindowDC(Wnd);
  BPP:=GetDeviceCaps(DC,BITSPIXEL);	
  releasedc(Wnd,DC);

  if BPP>=24 then
    begin
      flags2:=D3DX_SF_A8R8G8B8;
      num1:=0;
      flags1:=0;
      width1:=FBitmap1.width;
      height1:=FBitmap1.height;
      if XTexture1<>nil then XTexture1:=nil;
      if width1=0 then exit;
      if width1>height1 then height1:=width1 else width1:=height1;
      D3DXCreateTexture(Curr3DDevice,@flags1,@width1,@height1,flags2,mypallete,XTexture1,@num1);
      mypallete:=nil;
      SetTextureAlphaBitmap(XTexture1,FTransparency,FBitmap1);

    end else
    begin
      flags2:=D3DX_SF_A4R4G4B4;
      num1:=0;
      flags1:=0;
      width1:=FBitmap1.width;
      height1:=FBitmap1.height;
      if XTexture1<>nil then XTexture1:=nil;
      if width1=0 then exit;
      if width1>height1 then height1:=width1 else width1:=height1;
      D3DXCreateTexture(Curr3DDevice,@flags1,@width1,@height1,flags2,mypallete,XTexture1,@num1);
      mypallete:=nil;
      SetTextureAlphaBitmap(XTexture1,FTransparency,FBitmap1);

    end;
end;
procedure TIACubeIcon.Loaded;
begin
  inherited loaded;
  if Curr3DDevice<>nil then
    begin
      GetTexture1;
      GetTexture2;
    end;
  BaseMaterial1.emissive.r := FIntencity;
  BaseMaterial1.emissive.g := FIntencity;
  BaseMaterial1.emissive.b := FIntencity;
  BaseMaterial1.diffuse.a:=FTransparency;
  ISLoaded:=TRUE;
end;

procedure TIACubeIcon.SetIntencity(Value: Double);
begin
  if (Value>=0) and (Value<=1.0) then
    begin
      FIntencity := Value;
        BaseMaterial1.emissive.r := FIntencity;
        BaseMaterial1.emissive.g := FIntencity;
        BaseMaterial1.emissive.b := FIntencity;
        BaseMaterial1.diffuse.a:=FTransparency;
    end;
end;


procedure TIACubeIcon.SetBitmap2(Value: TBitmap);
begin
  if (Value=nil) or (Value.width=0) or (Value.height=0) then
    begin
      FBitmap2.width:=0;
      FBitmap2.height:=0;
      if Curr3DDevice<>nil then
      GetTexture2;
    end else
    begin
      FBitmap2.width:=Value.width;
      FBitmap2.height:=Value.height;
      FBitmap2.Canvas.Draw(0,0,Value);
      if Curr3DDevice<>nil then
      GetTexture2;
    end;
end;
procedure TIACubeIcon.GetTexture2;
  var
    flags1,width1,height1,num1 :DWord;
    flags2 : TD3DX_SurfaceFormat;
    mypallete : IDirectDrawPalette;
    Wnd : HWND;
    DC : HDC;
    BPP : Integer;
begin
  if Curr3DDevice=nil then exit;
  Wnd:=GetDesktopWindow;
  DC:=getwindowDC(Wnd);
  BPP:=GetDeviceCaps(DC,BITSPIXEL);	
  releasedc(Wnd,DC);

  if BPP>=24 then
    begin
      flags2:=D3DX_SF_A8R8G8B8;
      num1:=0;
      flags1:=0;
      width1:=FBitmap2.width;
      height1:=FBitmap2.height;
      if XTexture2<>nil then XTexture2:=nil;
      if width1=0 then exit;
      if width1>height1 then height1:=width1 else width1:=height1;
      D3DXCreateTexture(Curr3DDevice,@flags1,@width1,@height1,flags2,mypallete,XTexture2,@num1);
      mypallete:=nil;
      SetTextureAlphaBitmap(XTexture2,FTransparency,FBitmap2);

    end else
    begin
      flags2:=D3DX_SF_A4R4G4B4;
      num1:=0;
      flags1:=0;
      width1:=FBitmap2.width;
      height1:=FBitmap2.height;
      if XTexture2<>nil then XTexture2:=nil;
      if width1=0 then exit;
      if width1>height1 then height1:=width1 else width1:=height1;
      D3DXCreateTexture(Curr3DDevice,@flags1,@width1,@height1,flags2,mypallete,XTexture2,@num1);
      mypallete:=nil;
      SetTextureAlphaBitmap(XTexture2,FTransparency,FBitmap2);

    end;
end;  

procedure TIACubeIcon.SetBackColor(Value: TColor);
begin
  if FBackColor<>Value then
    begin
      FBackColor := Value;
    end;
end;

procedure TIACubeIcon.SetBackBitmap(Value: TBitmap);
begin
  if (Value=nil) or (Value.width=0) or (Value.height=0) then
    begin
      FBackBitmap.width:=0;
      FBackBitmap.height:=0;
    end else
    begin
      FBackBitmap.width:=Value.width;
      FBackBitmap.height:=Value.height;
      FBackBitmap.Canvas.Draw(0,0,Value);
    end;
end;

procedure TIACubeIcon.SetTransparency(Value: Double);
begin
  if (Value>=0) and (Value<=1.0) then
    begin
      FTransparency := Value;
      BaseMaterial1.emissive.r := FIntencity;
      BaseMaterial1.emissive.g := FIntencity;
      BaseMaterial1.emissive.b := FIntencity;
      BaseMaterial1.diffuse.a:=FTransparency;
      GetTexture1;
      GetTexture2;
    end;
end;
{procedure TIACubeIcon.SetTextureAlpha(var Atex: IDirectDrawSurface7;Alpha1 : double);
var
  CurrData : TDDSURFACEDESC2;
  p1 : pByteArray;
  i,j,step : integer;
  a16 : PWord;
  a32 : PCardinal;
begin
  if Atex=nil then exit;
  if alpha1<0 then alpha1:=0;
  if alpha1>1 then alpha1:=1;
  CurrData.dwSize:=sizeof(TDDSURFACEDESC2);
  Atex.Lock(nil,CurrData,DDLOCK_WAIT or DDLOCK_SURFACEMEMORYPTR,0);

  if ((DDPF_RGB and CurrData.ddpfPixelFormat.dwFlags)<>0) and
     ((DDPF_ALPHAPIXELS  and CurrData.ddpfPixelFormat.dwFlags)<>0)  and
     (CurrData.ddpfPixelFormat.dwRGBBitCount>=16)then
    begin
      LastPixelLen:=CurrData.ddpfPixelFormat.dwRGBBitCount;
      step:=CurrData.ddpfPixelFormat.dwRGBBitCount div 8;
      for i:=0 to CurrData.dwHeight-1 do
        begin
          p1:=pByteArray(integer(CurrData.lpSurface)+i*integer(CurrData.lPitch));
          for j:=0 to  CurrData.dwWidth-1 do
            begin
              case step of
                2 :
                  begin
                    a16:=PWord(integer(p1)+j*step);
                    if CurrData.ddpfPixelFormat.dwRGBAlphaBitMask=$F000 then
                    a16^:=(a16^ and not CurrData.ddpfPixelFormat.dwRGBAlphaBitMask) or (CurrData.ddpfPixelFormat.dwRGBAlphaBitMask and (round(Alpha1*255*256)));
                  end;
                3 :
                  begin

                  end;
                4 :
                  begin
                    a32:=PCardinal(integer(p1)+j*step);
                    if CurrData.ddpfPixelFormat.dwRGBAlphaBitMask=$FF000000 then
                    a32^:=(a32^ and not CurrData.ddpfPixelFormat.dwRGBAlphaBitMask) or (CurrData.ddpfPixelFormat.dwRGBAlphaBitMask and (round(Alpha1*255*256*256*256)));
                  end;    
              end;    
            end;
        end;
    end else
    begin
      Showmessage('Sorry no alpha channel defined');
    end;
  Atex.Unlock(nil);
end; }
procedure TIACubeIcon.SetTextureAlphaBitmap(var Atex: IDirectDrawSurface7;Alpha1 : double;ABitmap : TBitmap);
var
  CurrData : TDDSURFACEDESC2;
  p1,p2 : pByteArray;
  i,j,step : integer;
  a16 : PWord;
  a32 : PCardinal;
  B1 : TBitmap;
begin
  if Atex=nil then exit;
  if alpha1<0 then alpha1:=0;
  if alpha1>1 then alpha1:=1;
  CurrData.dwSize:=sizeof(TDDSURFACEDESC2);
  Atex.Lock(nil,CurrData,DDLOCK_WAIT or DDLOCK_SURFACEMEMORYPTR,0);

  if ((DDPF_RGB and CurrData.ddpfPixelFormat.dwFlags)<>0) and
     ((DDPF_ALPHAPIXELS  and CurrData.ddpfPixelFormat.dwFlags)<>0)  and
     (CurrData.ddpfPixelFormat.dwRGBBitCount>=16)then
    begin
      LastPixelLen:=CurrData.ddpfPixelFormat.dwRGBBitCount;
      B1:=TBitmap.create;
      B1.PixelFormat:=pf24Bit;
      B1.width:=CurrData.dwWidth;
      B1.Height:=CurrData.dwHeight;
      B1.Canvas.StretchDraw(rect(0,0,B1.width,B1.Height),ABitmap);


      step:=CurrData.ddpfPixelFormat.dwRGBBitCount div 8;
      for i:=0 to CurrData.dwHeight-1 do
        begin
          p1:=pByteArray(integer(CurrData.lpSurface)+i*integer(CurrData.lPitch));
          p2:=B1.ScanLine[i];
          for j:=0 to  CurrData.dwWidth-1 do
            begin
              case step of
                2 :
                  begin
                    a16:=PWord(integer(p1)+j*step);
                    if CurrData.ddpfPixelFormat.dwRGBAlphaBitMask=$F000 then
                    begin
                    a16^:=(a16^ and not CurrData.ddpfPixelFormat.dwRGBAlphaBitMask) or (CurrData.ddpfPixelFormat.dwRGBAlphaBitMask and (round(Alpha1*255*256)));
                    a16^:=(a16^ and not CurrData.ddpfPixelFormat.dwRBitMask) or (CurrData.ddpfPixelFormat.dwRBitMask and ((p2[j*3+2]*16)));
                    a16^:=(a16^ and not CurrData.ddpfPixelFormat.dwGBitMask ) or (CurrData.ddpfPixelFormat.dwGBitMask and ((p2[j*3+1])));
                    a16^:=(a16^ and not CurrData.ddpfPixelFormat.dwBBitMask ) or (CurrData.ddpfPixelFormat.dwBBitMask and ((p2[j*3+0] div 16)));
                    end;
                  end;
                3 :
                  begin

                  end;
                4 :
                  begin
                    a32:=PCardinal(integer(p1)+j*step);
                    if CurrData.ddpfPixelFormat.dwRGBAlphaBitMask=$FF000000 then
                    begin
                    a32^:=(a32^ and not CurrData.ddpfPixelFormat.dwRGBAlphaBitMask) or (CurrData.ddpfPixelFormat.dwRGBAlphaBitMask and (round(Alpha1*255*256*256*256)));
                    a32^:=(a32^ and not CurrData.ddpfPixelFormat.dwRBitMask) or (CurrData.ddpfPixelFormat.dwRBitMask and ((p2[j*3+2]*256*256)));
                    a32^:=(a32^ and not CurrData.ddpfPixelFormat.dwGBitMask ) or (CurrData.ddpfPixelFormat.dwGBitMask and ((p2[j*3+1]*256)));
                    a32^:=(a32^ and not CurrData.ddpfPixelFormat.dwBBitMask ) or (CurrData.ddpfPixelFormat.dwBBitMask and ((p2[j*3+0])));
                    end;
                  end;
              end;    
            end;
        end;
      B1.Free;  
    end else
    begin
      Showmessage('Sorry no alpha channel defined');
    end;
  Atex.Unlock(nil);
end;
procedure TIACubeIcon.SetShowRibers(Value: Boolean);
begin
  FShowRibers := Value;
end;

procedure TIACubeIcon.SetRibersColor(Value: TColor);
var
  p : pByte;
begin
  if (Value<>FRibersColor) then
    begin
      FRibersColor := Value;
      p:=pByte(@Value);
      RibersMaterial.emissive.r := p^/256;
      p:=pByte(integer(p)+1);
      RibersMaterial.emissive.g :=  p^/256;
      p:=pByte(integer(p)+1);
      RibersMaterial.emissive.b :=  p^/256;
      RibersMaterial.diffuse.a:=0.0;
    end;
end;

procedure TIACubeIcon.SetBouncing( Value: Boolean);
begin
  if FBouncing<>Value then  FBouncing := Value;
end;

procedure TIACubeIcon.SetBaseSize( Value: Double);
begin
  if (Value>=0.1) and (Value<=10) then
    begin
  FBaseSize := Value;
  D3DXMatrixPerspectiveLH(CurrProjectionMatrix,1*FBaseSize,1*FBaseSize,6,24);
  if Curr3DDevice<>nil then
  Curr3DDevice.SetTransform( D3DTRANSFORMSTATE_PROJECTION, CurrProjectionMatrix);
    end; 
end;

procedure TIACubeIcon.SetIconSizeCoeffitient(Value: Double);
var  dW,dH : integer;
begin

  if (Value>0.1) and (Value<10) then
    begin
      FIconSizeCoeffitient := Value;
      if ISLoaded then
        begin
      if Owner is TForm Then
        begin
          dw:=((Owner as TForm).width-(Owner as TForm).clientwidth) div 2;
          dh:=(Owner as TForm).height-(Owner as TForm).clientheight-dw;
          AIconSize:=round((dh-dw div 2)*FIconSizeCoeffitient);
          IConRect:=rect(dw div 2,dw div 2,dw div 2+AIconSize,dw div 2+AIconSize);
        end;
      DeleteX;
      InitX;
      if Curr3DDevice<>nil then
        begin
      GetTexture1;
      GetTexture2;
        end;
      BaseMaterial1.emissive.r := FIntencity;
      BaseMaterial1.emissive.g := FIntencity;
      BaseMaterial1.emissive.b := FIntencity;
      BaseMaterial1.diffuse.a:=FTransparency;
      if Owner is TForm Then
        begin
          RedrawWindow((Owner as TForm).handle,nil,0,RDW_INVALIDATE or RDW_ERASE or RDW_FRAME or
                                                     RDW_ERASENOW or RDW_UPDATENOW or RDW_ALLCHILDREN);
        end;
        end;
    end;
end;



function TIACubeIcon.AntiColor(AColor: TColor): TColor;
var
  pR,pB,pRr,pBr : pByte;
begin
  Result:=AColor;
  pR:=pByte(@AColor);
  pB:=pByte(integer(pR)+2);
  pRr:=pByte(@Result);
  pBr:=pByte(integer(pRr)+2);
  pRr^:=pB^;
  pBr^:=pR^;
end;

function TIACubeIcon.UNVisiblePartofRect(const ARect: TRect): HRGN;
var
  RGN1 : HRGN;
begin
  if RectVisible(MainDC,ARect) then
    begin
      if (ARect.right-ARect.Left)<=2 then
        begin
          if (ARect.Bottom-Arect.Top)<=2 then
            begin
              Result:=0;//NoChanges...
            end else
            begin
              Result:=0;
              RGN1:=UNVisiblePartofRect(rect(ARect.left,
                                             ARect.Top,
                                             ARect.Right,
                                             ARect.Top+(ARect.bottom-ARect.top) div 2
              ));
              if RGN1<>0 then
                begin
                  if result=0 then result:=CreateREctRGN(0,0,0,0);
                  combinergn(result,result,RGN1,RGN_OR);
                  deleteobject(RGN1);
                end;

              //\
              RGN1:=UNVisiblePartofRect(rect(ARect.left,
                                             ARect.Top+(ARect.bottom-ARect.top) div 2,
                                             ARect.Right,
                                             ARect.Bottom
              ));
              if RGN1<>0 then
                begin
                  if result=0 then result:=CreateREctRGN(0,0,0,0);
                  combinergn(result,result,RGN1,RGN_OR);
                  deleteobject(RGN1);
                end;
              
            end;
        end else
        begin
          if (ARect.Bottom-Arect.Top)<=2 then
            begin
              Result:=0;
              RGN1:=UNVisiblePartofRect(rect(ARect.left,
                                             ARect.Top,
                                             ARect.left+(ARect.right-ARect.left) div 2,
                                             ARect.Bottom
              ));
              if RGN1<>0 then
                begin
                  if result=0 then result:=CreateREctRGN(0,0,0,0);
                  combinergn(result,result,RGN1,RGN_OR);
                  deleteobject(RGN1);
                end;
              //
              RGN1:=UNVisiblePartofRect(rect(ARect.left+(ARect.right-ARect.left) div 2,
                                             ARect.Top,
                                             ARect.Right,
                                             ARect.Bottom
              ));
              if RGN1<>0 then
                begin
                  if result=0 then result:=CreateREctRGN(0,0,0,0);
                  combinergn(result,result,RGN1,RGN_OR);
                  deleteobject(RGN1);
                end;

            end else
            begin
              Result:=0;
              RGN1:=UNVisiblePartofRect(rect(ARect.left,
                                             ARect.Top,
                                             ARect.left+(ARect.right-ARect.left) div 2,
                                             ARect.Top+(ARect.bottom-ARect.top) div 2
              ));
              if RGN1<>0 then
                begin
                  if result=0 then result:=CreateREctRGN(0,0,0,0);
                  combinergn(result,result,RGN1,RGN_OR);
                  deleteobject(RGN1);
                end;
              //
              RGN1:=UNVisiblePartofRect(rect(ARect.left+(ARect.right-ARect.left) div 2,
                                             ARect.Top,
                                             ARect.Right,
                                             ARect.Top+(ARect.bottom-ARect.top) div 2
              ));
              if RGN1<>0 then
                begin
                  if result=0 then result:=CreateREctRGN(0,0,0,0);
                  combinergn(result,result,RGN1,RGN_OR);
                  deleteobject(RGN1);
                end;
              //\
              RGN1:=UNVisiblePartofRect(rect(ARect.left,
                                             ARect.Top+(ARect.bottom-ARect.top) div 2,
                                             ARect.left+(ARect.right-ARect.left) div 2,
                                             ARect.Bottom
              ));
              if RGN1<>0 then
                begin
                  if result=0 then result:=CreateREctRGN(0,0,0,0);
                  combinergn(result,result,RGN1,RGN_OR);
                  deleteobject(RGN1);
                end;
              //
              RGN1:=UNVisiblePartofRect(rect(ARect.left+(ARect.right-ARect.left) div 2,
                                             ARect.Top+(ARect.bottom-ARect.top) div 2,
                                             ARect.Right,
                                             ARect.Bottom
              ));
              if RGN1<>0 then
                begin
                  if result=0 then result:=CreateREctRGN(0,0,0,0);
                  combinergn(result,result,RGN1,RGN_OR);
                  deleteobject(RGN1);
                end;
            end;
        end;
    end else
    begin
      Result:=CreateRectRGN(ARect.left,ARect.Top,ARect.Right,ARect.Bottom);
    end;
end;

function TIACubeIcon.GetVisiblePartofRect(const ARect: TRect): HRGN;
var RGN1 : HRGN;
begin
  MainDC:=getwindowdc((Owner as TForm).handle);
  Result:=CreateRectRGN(ARect.left,ARect.Top,ARect.Right,ARect.Bottom);
  RGN1:=UNVisiblePartofRect(ARect);
  if RGN1<>0 then
    begin
      CombineRGN(Result,Result,RGN1,RGN_DIFF);
      Deleteobject(RGN1);
    end;
  REleaseDC((Owner as TForm).handle,MainDC);
end;


procedure TIACubeIcon.SetRotated(Value: boolean);
begin
  if Value<>FRotated then
    begin
      FRotated := Value;
    end;
end;

procedure TIACubeIcon.SetXrotation(Value: double);
begin
  if abs(Value)<1.0 then
  FXrotation := Value;
end;

procedure TIACubeIcon.SetYRotation(Value: Double);
begin
  if abs(Value)<1.0 then
  FYRotation := Value;
end;

procedure TIACubeIcon.SetZrotation(Value: double);
begin
  if abs(Value)<1.0 then
  FZrotation := Value;
end;

end.
