//---------------Library TIA Regions Foundation Classes--------
//_____________________________________________________________
//Delphi 4 created, not dynamic arrays used so ought to work at any version....
//_____________________License_________________________________
//Free for any use, changing and purposes except selling this software(GNU license)
//___________________Short descroption_________________________
// TIAregionFC - class encapsulated Windows Region with it data
//               fully automatization for using properties at new components
//               by this class (automatically saves/reads own data from DFM files)
//               Not eats resources so hold region in RGNDATA variant - not HRGN....
//               Saves/reads Region Data to/from files and streams.
// TIARegionListFC - class encapsulated List of TIAregionFC
//               fully automatization for using properties at new components
//               by this class (automatically saves/reads own data from DFM files)
//               Saves/reads Region Data to/from files and streams.
//               Full Automitized control of internal list of TIAregionFC - no need user control.
//
// RotateRegionFC  - rotated Any region on angle;
// ScaleRegionFC   - rescales Any region;
// InflateRegionFC - inflate/deflate Any region at  any border shift;
//
// CreateRectRGNAtAngleFC      - Create Rectangular Region rotated at angle aganist it center;
// CreateEllipticRGNAtAngleFC  - Create Elliptic Region rotated at angle aganist it center;
// CreateRoundRectRGNAtAngleFC - Create Rectangular Region with rounded Angles rotated at angle aganist it center;
// ________________________________________________________________
// Common rules for any objects/ functions in library:
//
// functions(even in objects) NOT eat Regions (HRGN) so you ought to destroy it manually - function deleteobject(RGN : HRGN);
// ________________________________________________________________
// ///////////////////////////////////////////////////////////////
// //Created by Andronov I.N www.geocities.com/andronovin/////////
// ///////////////////////////////////Good luck!!!////////////////
//------------------------------------------------------------
//--------------XXXXXXX ___ XX ___ XXXX-----------------------
//--------------XXXXXX /WWW\__/WW\ XXXX-----------------------
//--------------XXXXXX \@  @  WWW/ XXXX-----------------------
//--------------XXXXX O   WWWWWW| XXXXX-----------------------
//--------------XXXXXX ~_WWWWWW/ XXXXXX-----------------------
//--------------XXXXXXX  \WWWWW| XXXXXX-----------------------
// 30 july 2000...
unit iaregionsFC;

interface
uses Windows,Classes,Dialogs,SysUtils;
type
  TIAregionFC = class(TPersistent)
  private
    FVersionNumber : Cardinal;
    FIdentificator : Cardinal;
    FOwner: TComponent;
    FOnRegionChanged: TNotifyEvent;
    FRegionData: pRGNDATA;
    FDataSize : Cardinal;
    procedure SetOwner(Value: TComponent);
    procedure SetRegionData1(Value: pRGNDATA);

    function GetRegionData1: pRGNDATA;
    function GetRegion: HRGN;
    procedure SetRegion(Value: HRGN);
    function GetDataFullSize: Cardinal;
    function GetDataSize: Cardinal;
    function GetVersionNumber: Cardinal;
    function GetRegionNotEmpty: boolean;

  protected
    procedure AssignTo(Dest: TPersistent); override;
    procedure DefineProperties(Filer: TFiler); override;
    procedure WriteDataToStream(Stream: TStream); virtual;
    procedure ReadDataFromStream(Stream: TStream); virtual;
  public
    destructor Destroy; override;
    constructor Create(AOwner : TComponent); virtual;
    procedure Assign(Source: TPersistent); override;
    procedure WriteDataToFile(AFileName : string);
    procedure ReadDataFromFile(AFileName : string);
    procedure WriteRGNDataToStream(Stream: TStream);
    procedure ReadRGNDataFromStream(Stream: TStream);
    procedure ClearData;

    property Owner : TComponent read FOwner write SetOwner;
    property OnRegionChanged : TNotifyEvent read FOnRegionChanged write FOnRegionChanged;
    property RegionData :  pRGNDATA read GetRegionData1 write SetRegionData1;
    property Region : HRGN read GetRegion write SetRegion;
    property DataSize : Cardinal read GetDataSize;
    property DataFullSize : Cardinal read GetDataFullSize;
    property VersionNumber : Cardinal read GetVersionNumber;
    property RegionNotEmpty : boolean read GetRegionNotEmpty;
  end;
  PIAregionFC = ^TIAregionFC;
  TIARegionListFC = class(TPersistent)
  private
    FVersionNumber : Cardinal;
    FIdentificator : Cardinal;
    FOwner: TComponent;
    FInternalList : pInteger;
    FRegionsNumber : cardinal;
    FOnRegionListChanged: TNotifyEvent;
    procedure SetRegionsNumber(Value: cardinal);
    procedure SetOwner(Value: TComponent);
    procedure ClearList;
    function GetRegion(index: cardinal): TIAregionFC;
    procedure SetRegion(index: cardinal;Value: TIAregionFC);
  protected
    procedure AssignTo(Dest: TPersistent); override;
    procedure DefineProperties(Filer: TFiler); override;
    procedure WriteDataToStream(Stream: TStream); virtual;
    procedure ReadDataFromStream(Stream: TStream); virtual;
  public
    destructor Destroy; override;
    constructor Create(AOwner : TComponent); virtual;
    procedure Assign(Source: TPersistent); override;
    procedure AddRegion;
    procedure InsertRegion(BeforeIndex : cardinal);
    procedure DeleteRegion(Index : cardinal);
    procedure SwapRegions(Index1,Index2 : cardinal);
    procedure WriteRGNListDataToStream(Stream: TStream);
    procedure ReadRGNListDataFromStream(Stream: TStream);
    procedure WriteListDataToFile(AFileName : string);
    procedure ReadListDataFromFile(AFileName : string);

    property RegionsNumber : cardinal read FRegionsNumber write SetRegionsNumber;
    property Owner : TComponent read FOwner write SetOwner;
    property OnRegionListChanged : TNotifyEvent read FOnRegionListChanged write FOnRegionListChanged;
    property Region [index : cardinal] : TIAregionFC read GetRegion write SetRegion; default;
  end;
  PIARegionListFC = ^TIARegionListFC;


  procedure RotateRegionFC(var RGN : HRGN;Angle : Integer);
  procedure ScaleRegionFC(var RGN: HRGN;XScale,YScale : Double);
  procedure InflateRegionFC(var RGN : HRGN;Dx,Dy : integer);

  function CreateRectRGNAtAngleFC(ALeft,ATop,ARight,ABottom,Angle : integer): HRGN;
  function CreateEllipticRGNAtAngleFC(ALeft,ATop,ARight,ABottom,Angle : integer): HRGN;
  function CreateRoundRectRGNAtAngleFC(ALeft,ATop,ARight,ABottom,Angle : integer;Radius : Cardinal): HRGN;

implementation
// This functions gets the rotated region at angle in 0.1 of degrees...
procedure RotateRegionFC(var RGN : HRGN;Angle : Integer);
var R : TRect;
    x,y,xB,xE : integer;
    Tr,Tr1 : HRGN;
    cs,sn : Double;
    pt1 : array [1..4] of TPoint;
begin
  if RGN<>0 then
    begin
      Tr1:=CreateRectRGN(0,0,0,0);
      R:=Rect(0,0,0,0);
      GetRgnBox(RGN,R);
      cs:=cos(Angle*pi/1800);
      sn:=sin(Angle*pi/1800);
      for y:=R.Top to R.Bottom do
        begin
          xB:=R.Left;xE:=R.left;
          for x:=R.Left to R.Right do
            begin
              if PtInRegion(RGN,x,y) then
                begin
                  xE:=x;
                end else
                begin
                  if xB=xE then
                    begin
                      xB:=x;xE:=x;
                    end else
                    begin
                      pt1[1].x:=round(xB*cs+y*sn)-1;pt1[1].y:=round(-xB*sn+y*cs)-1;
                      pt1[2].x:=round(xE*cs+y*sn)+1;pt1[2].y:=round(-xE*sn+y*cs)-1;
                      pt1[3].x:=round(xE*cs+y*sn)+1;pt1[3].y:=round(-xE*sn+y*cs)+1;
                      pt1[4].x:=round(xB*cs+y*sn)-1;pt1[4].y:=round(-xB*sn+y*cs)+1;
                      Tr:=CreatePolygonRGN(pt1[1],4,Alternate);
                      ComBinergn(Tr1,Tr1,TR,RGN_OR);
                      deleteobject(Tr);
                      xB:=x;xE:=x;
                    end;
                end;
            end;
          if xB<>xE then
            begin
              pt1[1].x:=round(xB*cs+y*sn)-1;pt1[1].y:=round(-xB*sn+y*cs)-1;
              pt1[2].x:=round(xE*cs+y*sn)+1;pt1[2].y:=round(-xE*sn+y*cs)-1;
              pt1[3].x:=round(xE*cs+y*sn)+1;pt1[3].y:=round(-xE*sn+y*cs)+1;
              pt1[4].x:=round(xB*cs+y*sn)-1;pt1[4].y:=round(-xB*sn+y*cs)+1;
              Tr:=CreatePolygonRGN(pt1[1],4,Alternate);
              ComBinergn(Tr1,Tr1,TR,RGN_OR);
              deleteobject(Tr);
            end;
        end;
      combinergn(RGN,Tr1,0,RGN_COPY);
      deleteobject(TR1);  
    end;
end;
//Next procedure rescales the region ...
procedure ScaleRegionFC(var RGN: HRGN;XScale,YScale : Double);
var myregion : TIAregionFC;
    myForm : XFORM;
begin
  if RGN<>0 then
    begin
      myregion:=TIAregionFC.Create(nil);
      myregion.Region:=RGN;
      myFORM.eDx:=0;
      myFORM.eDy:=0;
      myFORM.eM11:=XScale;
      myform.eM22:=YScale;
      myform.eM12:=0;
      myform.eM21:=0;
      deleteobject(RGN);
      RGN:=ExtCreateRegion(@myform,myregion.Fdatasize-sizeof(myregion.FRegionData^),myregion.FRegionData^);
      myregion.free;
    end;
end;
//procedure inflates or deflates regions....
procedure InflateRegionFC(var RGN : HRGN;Dx,Dy : integer);
var R : TRect;
    x,y,xB,xE,yB,YE,xxB,xxE,yyB,yyE : integer;
    Tr,Tr1,Tr2 : HRGN;
begin
  if RGN<>0 then
    begin
      Tr1:=CreateRectRGN(0,0,0,0);
      Tr2:=CreateRectRGN(0,0,0,0);
      R:=Rect(0,0,0,0);
      GetRgnBox(RGN,R);
      for y:=R.Top to R.Bottom do
        begin
          xB:=R.Left;xE:=R.left;
          for x:=R.Left to R.Right do
            begin
              if PtInRegion(RGN,x,y) then
                begin
                  xE:=x;
                end else
                begin
                  if xB=xE then
                    begin
                      xB:=x;xE:=x;
                    end else
                    begin
                      xxB:=xB-Dx;xxE:=xE+Dx;
                      if xxB<xxE then
                        begin
                          Tr:=CreateRectRGN(xxB,y,xxE+1,y+1);
                          ComBinergn(Tr1,Tr1,TR,RGN_OR);
                          deleteobject(Tr);
                        end;
                      xB:=x;xE:=x;
                    end;
                end;
            end;
          if xB<>xE then
            begin
              xxB:=xB-Dx;xxE:=xE+Dx;
              if xxB<xxE then
                begin
                  Tr:=CreateRectRGN(xxB,y,xxE+1,y+1);
                  ComBinergn(Tr1,Tr1,TR,RGN_OR);
                  deleteobject(Tr);
                end;
            end;
        end;
      //--
      R:=Rect(0,0,0,0);
      GetRgnBox(TR1,R);
      for x:=R.Left to R.Right do
        begin
          yB:=R.Top;yE:=R.Top;
          for y:=R.Top to R.Bottom do
            begin
              if PtInRegion(Tr1,x,y) then
                begin
                  yE:=y;
                end else
                begin
                  if yB=yE then
                    begin
                      yB:=y;yE:=y;
                    end else
                    begin
                      yyB:=yB-Dy;yyE:=yE+Dy;
                      if yyB<yyE then
                        begin
                          Tr:=CreateRectRGN(x,yyB,x+1,yyE+1);
                          ComBinergn(Tr2,Tr2,TR,RGN_OR);
                          deleteobject(Tr);
                        end;
                      yB:=y;yE:=y;
                    end;
                end;
            end;
          if yB<>yE then
            begin
              yyB:=yB-Dy;yyE:=yE+Dy;
              if yyB<yyE then
                begin
                  Tr:=CreateRectRGN(x,yyB,x+1,yyE+1);
                  ComBinergn(Tr2,Tr2,TR,RGN_OR);
                  deleteobject(Tr);
                end;
            end;
        end;

      combinergn(RGN,Tr2,0,RGN_COPY);
      deleteobject(TR1);
      deleteobject(TR2);
    end;
end;
//This function create rect region roteted at angle aganist it center....
function CreateRectRGNAtAngleFC(ALeft,ATop,ARight,ABottom,Angle : integer): HRGN;
var
  p1 : array [1..4] of Tpoint;
  xc,yc,sn,cs : double;
begin
  Xc:=(ALeft+ARight)/2;
  Yc:=(ATop+ABottom)/2;
  sn:=sin(Angle*pi/1800);
  cs:=cos(Angle*pi/1800);
  p1[1].x:=round((ALeft-Xc)*cs+Xc+(ATop-Yc)*sn);
  p1[1].y:=round(-(ALeft-Xc)*sn+(ATop-Yc)*cs+Yc);
  p1[2].x:=round((ARight-Xc)*cs+Xc+(ATop-Yc)*sn);
  p1[2].y:=round(-(ARight-Xc)*sn+(ATop-Yc)*cs+Yc);
  p1[3].x:=round((ARight-Xc)*cs+Xc+(ABottom-Yc)*sn);
  p1[3].y:=round(-(ARight-Xc)*sn+(ABottom-Yc)*cs+Yc);
  p1[4].x:=round((ALeft-Xc)*cs+Xc+(ABottom-Yc)*sn);
  p1[4].y:=round(-(ALeft-Xc)*sn+(ABottom-Yc)*cs+Yc);
  result:=CreatePolygonRGN(p1[1],4,Alternate);
end;
//This function create elliptic region roteted at angle aganist it center....
function CreateEllipticRGNAtAngleFC(ALeft,ATop,ARight,ABottom,Angle : integer): HRGN;
var
  p1,p2 : PPoint;
  dx,dy,N :  integer;
  xc,yc,sn,cs,x,y : double;
  i : cardinal;
begin
  Xc:=(ALeft+ARight)/2;
  Yc:=(ATop+ABottom)/2;
  sn:=sin(Angle*pi/1800);
  cs:=cos(Angle*pi/1800);
  dx:=abs(ARight-ALeft);
  dy:=abs(ABottom-ATop);
  N:=dx+dy;if N<8 then N:=8;
  getmem(p1,8*N);
  for i:=0 to N-1 do
    begin
      x:=dx*cos(i*pi*2/N)/2;
      y:=dy*sin(i*pi*2/N)/2;
      p2:=PPoint(cardinal(p1)+i*8);
      p2^.x:=round(x*cs+Xc+y*sn);
      p2^.y:=round(-x*sn+y*cs+Yc);
    end;
  result:=CreatePolygonRGN(p1^,N,Alternate);
  freemem(p1,8*N);
end;
//This function create rounrect region roteted at angle aganist it center....
function CreateRoundRectRGNAtAngleFC(ALeft,ATop,ARight,ABottom,Angle : integer;Radius : Cardinal): HRGN;
var
  p1,p2 : PPoint;
  dx,dy,N :  integer;
  xc,yc,sn,cs,x,y,xcc,ycc : double;
  i : cardinal;
begin

  Xc:=(ALeft+ARight)/2;
  Yc:=(ATop+ABottom)/2;

  sn:=sin(Angle*pi/1800);
  cs:=cos(Angle*pi/1800);
  dx:=abs(ARight-ALeft);
  dy:=abs(ABottom-ATop);
  Xcc:=dx/2;
  Ycc:=dy/2;
  if Radius>cardinal(dx div 2) then Radius:=dx div 2;
  if Radius>cardinal(dy div 2) then Radius:=dy div 2;
  if radius<=0 then
    begin
      result:=CreateRectRGNAtAngleFC(ALeft,ATop,ARight,ABottom,Angle);
      exit;
    end;
  N:=4*Radius+8;
  getmem(p1,8*N);
  //--
  x:=-Xcc+int64(Radius);
  y:=0-Ycc;
  p2:=PPoint(cardinal(p1)+0*8);
  p2^.x:=round(x*cs+Xc+y*sn);
  p2^.y:=round(-x*sn+y*cs+Yc);
  x:=Xcc-int64(Radius);
  y:=0-Ycc;
  p2:=PPoint(cardinal(p1)+1*8);
  p2^.x:=round(x*cs+Xc+y*sn);
  p2^.y:=round(-x*sn+y*cs+Yc);
  for i:=2 to 2+radius-1 do
    begin
      x:=Xcc-int64(Radius) + radius*sin((i-2)*pi/(2*radius));
      y:=-Ycc+int64(radius)- radius*cos((i-2)*pi/(2*radius));
      p2:=PPoint(cardinal(p1)+i*8);
      p2^.x:=round(x*cs+Xc+y*sn);
      p2^.y:=round(-x*sn+y*cs+Yc);
    end;
  //--
  x:=Xcc;
  y:=radius-Ycc;
  p2:=PPoint(cardinal(p1)+(2+radius)*8);
  p2^.x:=round(x*cs+Xc+y*sn);
  p2^.y:=round(-x*sn+y*cs+Yc);
  x:=Xcc;
  y:=-int64(radius)+Ycc;
  p2:=PPoint(cardinal(p1)+(2+radius+1)*8);
  p2^.x:=round(x*cs+Xc+y*sn);
  p2^.y:=round(-x*sn+y*cs+Yc);
  for i:=4+radius to 4+2*radius-1 do
    begin
      x:=Xcc-int64(radius)+ radius*cos((i-4-radius)*pi/(2*radius));
      y:=Ycc-int64(radius)+ radius*sin((i-4-radius)*pi/(2*radius));
      p2:=PPoint(cardinal(p1)+i*8);
      p2^.x:=round(x*cs+Xc+y*sn);
      p2^.y:=round(-x*sn+y*cs+Yc);
    end;
  //--
  x:=Xcc-int64(radius);
  y:=Ycc;
  p2:=PPoint(cardinal(p1)+(4+2*radius)*8);
  p2^.x:=round(x*cs+Xc+y*sn);
  p2^.y:=round(-x*sn+y*cs+Yc);
  x:=-Xcc+int64(Radius);
  y:=Ycc;
  p2:=PPoint(cardinal(p1)+(4+2*radius+1)*8);
  p2^.x:=round(x*cs+Xc+y*sn);
  p2^.y:=round(-x*sn+y*cs+Yc);
  for i:=6+2*radius to 6+3*radius-1 do
    begin
      x:=-Xcc+radius         - radius*sin((i-6-2*radius)*pi/(2*radius));
      y:=Ycc-int64(radius)   + radius*cos((i-6-2*radius)*pi/(2*radius));
      p2:=PPoint(cardinal(p1)+i*8);
      p2^.x:=round(x*cs+Xc+y*sn);
      p2^.y:=round(-x*sn+y*cs+Yc);
    end;
    //--
  x:=-Xcc;
  y:=Ycc-int64(radius);
  p2:=PPoint(cardinal(p1)+(6+3*radius)*8);
  p2^.x:=round(x*cs+Xc+y*sn);
  p2^.y:=round(-x*sn+y*cs+Yc);
  x:=-Xcc;
  y:=-Ycc+int64(Radius);
  p2:=PPoint(cardinal(p1)+(6+3*radius+1)*8);
  p2^.x:=round(x*cs+Xc+y*sn);
  p2^.y:=round(-x*sn+y*cs+Yc);
  for i:=8+3*radius to 8+4*radius-1 do
    begin
      x:=-Xcc+radius   - radius*cos((i-8-3*radius)*pi/(2*radius));
      y:=-Ycc+radius   - radius*sin((i-8-3*radius)*pi/(2*radius));
      p2:=PPoint(cardinal(p1)+i*8);
      p2^.x:=round(x*cs+Xc+y*sn);
      p2^.y:=round(-x*sn+y*cs+Yc);
    end;        
  result:=CreatePolygonRGN(p1^,N,Alternate);
  freemem(p1,8*N);
end;
{ TIAregionFC }

procedure TIAregionFC.Assign(Source: TPersistent);
begin
  if Source is TIAregionFC then
    begin
      TIAregionFC(Source).AssignTo(self);
    end else
    begin
      inherited Assign(Source);//This will create error at default;
    end;
end;

procedure TIAregionFC.AssignTo(Dest: TPersistent);
begin
  if Dest is TIAregionFC then
    begin
      TIAregionFC(Dest).RegionData:=self.FRegionData;
    end else
    begin
      inherited AssignTo(Dest);//This will  create error at default;
    end;
end;

procedure TIAregionFC.ClearData;
begin
  freemem(FRegionData,FDataSize);
  Fdatasize:=0;
  FRegionData:=nil;
end;

constructor TIAregionFC.Create(AOwner: TComponent);
begin
  inherited Create;
  FVersionNumber:=1;
  FIdentificator:=131313;
  FOwner:=AOwner;
  FDataSize:=0;
  FRegionData:=nil;
end;

procedure TIAregionFC.DefineProperties(Filer: TFiler);
begin
  inherited DefineProperties(Filer);//No action at default;
  Filer.DefineBinaryProperty('RegionData',ReadDataFromStream,WriteDataToStream,TRUE);
end;

destructor TIAregionFC.Destroy;
begin
  ClearData;
  inherited Destroy;
end;


function TIAregionFC.GetDataFullSize: Cardinal;
begin
  result:=FDataSize;
end;

function TIAregionFC.GetDataSize: Cardinal;
begin
  result:=(pInteger(cardinal(FRegionData)+12))^;
end;

function TIAregionFC.GetRegion: HRGN;
var
  myForm : XFORM;
begin
  if FDataSize=0 then
    begin
      result:=0;
    end else
    begin
      myFORM.eDx:=0;
      myFORM.eDy:=0;
      myFORM.eM11:=1;
      myform.eM22:=1;
      myform.eM12:=0;
      myform.eM21:=0;
      result:=ExtCreateRegion(@myform,Fdatasize-sizeof(FRegionData^),FRegionData^);
    end;
end;

function TIAregionFC.GetRegionData1: pRGNDATA;
begin
  if FDataSize<>0 then result:=FRegionData else result:=nil;
end;

function TIAregionFC.GetRegionNotEmpty: boolean;
begin
  result:=false;
  if FDataSize<>0 then
    begin
      if not ((FRegionData.rdh.rcBound.Left=FRegionData.rdh.rcBound.right) or
             (FRegionData.rdh.rcBound.top=FRegionData.rdh.rcBound.bottom)) then result:= true;
    end;
end;

function TIAregionFC.GetVersionNumber: Cardinal;
begin
  result:=FVersionNumber;
end;

procedure TIAregionFC.ReadDataFromFile(AFileName: string);
var Stream : TFileStream;
begin
  try
    Stream :=TFileStream.Create(AFileName,fmOpenRead);
    try
      ReadDataFromStream(Stream);
    finally
      Stream.Free;
    end;
  except
    ShowMessage('Can''t open file to read'+AFileName);
  end;
end;

procedure TIAregionFC.ReadDataFromStream(Stream: TStream);
var
   ID,VN : Cardinal;
begin
  Stream.Read(ID,4);
  if  FIdentificator<>ID then
    begin
      ShowMessage('This is not TIAregionFC!!!');
      Exit;
    end;
  Stream.Read(VN,4);
  if VN>FVersionNumber then
    begin
      ShowMessage('Detected Version ' +inttostr(VN)+' of TIAregionFC!!! More than current('+inttostr(FVersionNumber)+') can''t read.');
      Exit;
    end else
    begin
      if VN<FVersionNumber then
        begin
          exit;//here will be code to conversion of old data to new one...
        end;
    end;
  ClearData;
  Stream.Read(FDataSize,4);
  if FDataSize>0 then
    begin
      getmem(FRegionData,FDataSize);
      Stream.read(FRegionData^,FDataSize);
    end;
end;

procedure TIAregionFC.ReadRGNDataFromStream(Stream: TStream);
begin
  ReadDataFromStream(Stream);
end;

procedure TIAregionFC.SetOwner(Value: TComponent);
begin
  FOwner := Value;
end;

procedure TIAregionFC.SetRegion(Value: HRGN);
var NewDataSize : Cardinal;
begin
  if FDataSize<>0 then ClearData;
  if Value<>0 then
    begin
      NewDataSize:=sizeof(FRegionData^)+getregiondata(Value,0,nil);
      getmem(FRegionData,NewDataSize);
      getregiondata(Value,NewDataSize-sizeof(FRegionData^),FRegionData);
      Fdatasize:=NewDataSize;
    end;
  if @FOnRegionChanged<>nil then FOnRegionChanged(self);  
end;

procedure TIAregionFC.SetRegionData1(Value: pRGNDATA);
var newdatasize : cardinal;
begin
  if FDataSize<>0 then ClearData;
  if Value<>nil then
    begin
      newdatasize:=sizeof(FRegionData^)+(pInteger(Value))^+(pInteger(cardinal(Value)+12))^;
      getmem(FRegionData,NewDataSize);
      CopyMemory(FRegionData,Value,NewDataSize);
      Fdatasize:=NewDataSize;
    end;
  if @FOnRegionChanged<>nil then FOnRegionChanged(self);
end;

procedure TIAregionFC.WriteDataToFile(AFileName: string);
var Stream : TFileStream;
begin
  try
    Stream :=TFileStream.Create(AFileName,fmCreate);
    try
      WriteDataToStream(Stream);
    finally
      Stream.Free;
    end;
  except
    ShowMessage('Can''t create file '+AFileName);
  end;
end;

procedure TIAregionFC.WriteDataToStream(Stream: TStream);
begin
  Stream.Write(FIdentificator,4);
  Stream.Write(FVersionNumber,4);
  Stream.Write(FDataSize,4);
  if FDataSize>0 then Stream.Write(FRegionData^,FDataSize);
end;

procedure TIAregionFC.WriteRGNDataToStream(Stream: TStream);
begin
  WriteDataToStream(Stream);
end;

{ TIARegionListFC }

procedure TIARegionListFC.AddRegion;
begin
  self.RegionsNumber:=FRegionsNumber+1;
end;

procedure TIARegionListFC.Assign(Source: TPersistent);
begin
  if Source is TIARegionListFC then
    begin
      TIARegionListFC(Source).AssignTo(self);
    end else
    begin
      inherited Assign(Source);//This will create error at default;
    end;
end;

procedure TIARegionListFC.AssignTo(Dest: TPersistent);
var i : cardinal;
begin
  if Dest is TIARegionListFC then
    begin
      TIARegionListFC(Dest).RegionsNumber:=FRegionsNumber;
      if FRegionsNumber>0 then
        for i:=0 to FRegionsNumber-1 do
          begin
            TIARegionListFC(Dest).region[i]:=self.Region[i];
          end;
    end else
    begin
      inherited AssignTo(Dest);//This will  create error at default;
    end;
end;

procedure TIARegionListFC.ClearList;
var i : cardinal;
    tempregion : TIAregionFC;
begin
  if FRegionsNumber>0 then
    begin
      for i:=0 to FRegionsNumber-1 do
        begin
          tempregion:=(PIAregionFC(cardinal(FInternalList)+4*i))^;
          tempregion.Free;
        end;
      FInternalList:=nil;
      FRegionsNumber:=0;
    end;
end;

constructor TIARegionListFC.Create(AOwner: TComponent);
begin
  inherited Create;
  FInternalList:=nil;
  FRegionsNumber:=0;
  FOwner:=AOwner;
  FVersionNumber :=1;
  FIdentificator :=131312;
end;

procedure TIARegionListFC.DefineProperties(Filer: TFiler);
begin
  inherited DefineProperties(Filer);//No action at default;
  Filer.DefineBinaryProperty('RegionListData',ReadDataFromStream,WriteDataToStream,TRUE);
end;

procedure TIARegionListFC.DeleteRegion(Index: cardinal);
var tempP : pinteger;
    tempregion : TIAregionFC;
begin
  if index<FRegionsNumber then
    begin
      if FRegionsNumber=1 then
        begin
          ClearList;
          FRegionsNumber:=0;
        end else
        begin
          tempregion:=(PIAregionFC(cardinal(FInternalList)+4*Index))^;
          tempregion.Free;
          getmem(tempp,(FRegionsNumber-1)*4);
          if Index>0 then copymemory(tempp,FInternalList,4*Index);
          if Index<(FRegionsNumber-1) then copymemory(pointer(cardinal(tempp)+4*(Index)),
                                                      pointer(cardinal(FInternalList)+4*(Index+1)),
                                                      4*(FRegionsNumber-Index-1));
          freemem(FInternalList,4*FRegionsNumber);
          FInternalList:=tempp;
          FRegionsNumber:=FRegionsNumber-1;
        end;
    end else
    begin
      Showmessage('TIARegionListFC : Delete index out of bounds' + 'you ask for '+inttostr(index) + ' we have : '+inttostr(FRegionsNumber));
    end;
end;

destructor TIARegionListFC.Destroy;
begin
  ClearList;
  inherited Destroy;
end;

function TIARegionListFC.GetRegion(index: cardinal): TIAregionFC;
begin
  if FRegionsNumber>0 then
    begin
      if Index<FRegionsNumber then
      result:=(PIAregionFC(cardinal(FInternalList)+4*index))^
      else result:=nil;
    end else
    begin
      result:=nil;
    end;
end;

procedure TIARegionListFC.InsertRegion(BeforeIndex: cardinal);
var tempP : pinteger;
    tempregion : TIAregionFC;
begin
  if BeforeIndex<FRegionsNumber then
    begin
      getmem(tempp,(FRegionsNumber+1)*4);
      if BeforeIndex>0 then copymemory(tempp,FInternalList,4*BeforeIndex);
      copymemory(pointer(cardinal(tempp)+4*(BeforeIndex+1)),pointer(cardinal(FInternalList)+4*(BeforeIndex)),
                                                  4*(FRegionsNumber-BeforeIndex));
      freemem(FInternalList,4*FRegionsNumber);
      FInternalList:=tempp;
      tempregion := TIAregionFC.Create(FOwner);
      tempp:=pinteger(BeforeIndex*4+cardinal(FInternalList));
      tempp^:=integer(tempregion);
      FRegionsNumber:=FRegionsNumber+1;
    end else
    begin
      Showmessage('TIARegionListFC : Insert index out of bounds' + 'you ask for '+inttostr(BeforeIndex) + ' we have : '+inttostr(FRegionsNumber));
    end;
end;

procedure TIARegionListFC.ReadDataFromStream(Stream: TStream);
var
   ID,VN,NN,i : Cardinal;
begin
  Stream.Read(ID,4);
  if  FIdentificator<>ID then
    begin
      ShowMessage('This is not TIARegionListFC!!!');
      Exit;
    end;
  Stream.Read(VN,4);
  if VN>FVersionNumber then
    begin
      ShowMessage('Detected Version ' +inttostr(VN)+' of TIARegionListFC!!! More than current('+inttostr(FVersionNumber)+') can''t read.');
      Exit;
    end else
    begin
      if VN<FVersionNumber then
        begin
          exit;//here will be code to conversion of old data to new one...
        end;
    end;
  ClearList;
  Stream.Read(NN,4);
  self.RegionsNumber:=NN;
  if NN>0 then
    begin
      for i:=0 to NN-1 do
        begin
          self[i].ReadDataFromStream(Stream);
        end;
    end;
end;

procedure TIARegionListFC.ReadListDataFromFile(AFileName: string);
var Stream : TFileStream;
begin
  try
    Stream :=TFileStream.Create(AFileName,fmOpenRead);
    try
      ReadDataFromStream(Stream);
    finally
      Stream.Free;
    end;
  except
    ShowMessage('Can''t open file to read'+AFileName);
  end;
end;

procedure TIARegionListFC.ReadRGNListDataFromStream(Stream: TStream);
begin
  ReadDataFromStream(Stream);
end;

procedure TIARegionListFC.SetOwner(Value: TComponent);
begin
  FOwner := Value;
end;

procedure TIARegionListFC.SetRegion(index: cardinal;Value: TIAregionFC);
var tempregion : TIAregionFC;
begin
  if FRegionsNumber>0 then
    begin
      if Index<FRegionsNumber then
        begin
          tempregion:=(PIAregionFC(cardinal(FInternalList)+4*index))^;
          tempregion.Assign(Value);
        end else
        begin
          Showmessage('TIARegionListFC : Assign index out of bounds' + 'you ask for '+inttostr(index) + ' we have : '+inttostr(FRegionsNumber));
        end;
    end;
end;

procedure TIARegionListFC.SetRegionsNumber(Value: cardinal);
var tempP : pinteger;
    tempregion : TIAregionFC;
    i : cardinal;
begin
  if Value<>FRegionsNumber then
    begin
      if Value>FRegionsNumber then
        begin
          getmem(tempp,Value*4);
          if FRegionsNumber>0 then
            begin
              copymemory(tempp,FInternalList,4*FRegionsNumber);
              freemem(FInternalList,4*FRegionsNumber);
            end;  
          FInternalList:=tempp;
          for i:=FRegionsNumber to Value-1 do
            begin
              tempregion:=TIAregionFC.Create(FOwner);
              tempp:=pinteger(i*4+cardinal(FInternalList));
              tempp^:=integer(tempregion);
            end;
        end else
        begin
          if Value<>0 then
            begin
              getmem(tempp,Value*4);
              copymemory(tempp,FInternalList,4*Value);
              for i:=Value to FRegionsNumber-1 do
              begin
                tempregion:=(PIAregionFC(cardinal(FInternalList)+4*i))^;
                tempregion.Free;
              end;
              freemem(FInternalList,4*FRegionsNumber);
              FInternalList:=tempp;
            end else
            begin
              ClearList;
            end;
        end;
      FRegionsNumber:=Value;
      if @FOnRegionListChanged<>nil then FOnRegionListChanged(self);
    end;
end;

procedure TIARegionListFC.SwapRegions(Index1, Index2: cardinal);
var i : integer;
    i1 : pinteger;
begin
  if (index1<FRegionsNumber) and (index2<FRegionsNumber) then
    begin
      i:=pinteger(cardinal(FInternalList)+4*index1)^;
      i1:=pinteger(cardinal(FInternalList)+4*index1);
      i1^:=pinteger(cardinal(FInternalList)+4*index2)^;
      i1:=pinteger(cardinal(FInternalList)+4*index2);
      i1^:=i;
    end else
    begin
      showmessage('TIARegionListFC : Can''t swap ; indexes out of bounds, Try indexes : '
      +inttostr(index1)+' , '+ inttostr(index2)+'We have : '+ inttostr(FRegionsNumber));
    end;
end;

procedure TIARegionListFC.WriteDataToStream(Stream: TStream);
var i : cardinal;
begin
  Stream.Write(FIdentificator,4);
  Stream.Write(FVersionNumber,4);
  Stream.Write(FRegionsNumber,4);
  if FRegionsNumber>0 then
    begin
      for i:=0 to FRegionsNumber-1 do
        begin
          self[i].WriteDataToStream(Stream);
        end;
    end;
end;

procedure TIARegionListFC.WriteListDataToFile(AFileName: string);
var Stream : TFileStream;
begin
  try
    Stream :=TFileStream.Create(AFileName,fmCreate);
    try
      WriteDataToStream(Stream);
    finally
      Stream.Free;
    end;
  except
    ShowMessage('Can''t create file '+AFileName);
  end;
end;

procedure TIARegionListFC.WriteRGNListDataToStream(Stream: TStream);
begin
  WriteDataToStream(Stream);
end;

end.
